@@ -7,7 +7,7 @@ from typing import TYPE_CHECKING
 
 from convtools import conversion as c
 
-from ..base import BaseStep, TypeValueCodeGenArgs, _none
+from ..base import BaseStep, TypeValueCodeGenArgs
 from ..utils import is_generic_alias, is_model_cls
 
 
@@ -26,21 +26,6 @@ class BaseValidator(abc.ABC, BaseStep):
         """abstract validate method"""
 
 
-class Required(BaseValidator):
-    """Internally used validator, which checks fields without defaults for
-    existence"""
-
-    name = "required"
-    __slots__ = ()
-    _error = {"required": True}
-
-    def validate(self, field_name, data, errors):
-        if data is _none:
-            errors[field_name]["__ERRORS"] = self._error
-        else:
-            return True
-
-
 class Type(BaseValidator):
     """Type validator (also used internally). It runs isinstance(data, types)
     on input data and it doesn't support typing generics"""
